class DeviceCharacteristics:
    HardwareRevision = "2a27"
    FirmwareRevision = "2a26"


class CrownstoneCharacteristics:
    SessionData    = "24f0000e-7d10-4805-bfc1-7663a01c3bff"
    FactoryReset   = "24f00009-7d10-4805-bfc1-7663a01c3bff"
    Control        = "24f0000c-7d10-4805-bfc1-7663a01c3bff"
    Result         = "24f0000d-7d10-4805-bfc1-7663a01c3bff"


class SetupCharacteristics:
    MacAddress     = "24f10002-7d10-4805-bfc1-7663a01c3bff"
    SessionKey     = "24f10003-7d10-4805-bfc1-7663a01c3bff"
    GoToDFU        = "24f10006-7d10-4805-bfc1-7663a01c3bff"
    SessionData    = "24f1000e-7d10-4805-bfc1-7663a01c3bff"
    SetupControl   = "24f1000c-7d10-4805-bfc1-7663a01c3bff"
    Result         = "24f1000d-7d10-4805-bfc1-7663a01c3bff"

class DFUCharacteristics:
    ControlPoint   = "00001531-1212-EFDE-1523-785FEABCD123"
    Packet         = "00001532-1212-EFDE-1523-785FEABCD123"

